require "./permission"

# An Acl Path identify one or several resources based on joker `*`
# in order to match several files.
class Acl::Path
   getter value : String
   getter regex : Regex

   def self.value_to_regex(value : String) : Regex
     value_regex = value.gsub "*", ".*"
     Regex.new "^#{value_regex}$"
   end

   def initialize(@value : String)
     @regex = Acl::Path.value_to_regex @value
   end

   def match?(other_path : String) : Bool
     !!@regex.match other_path
   end

   def to_s : String
     @value
   end

   def to_s(io : IO)
     io << @value
   end

   EXT_SIZE = ".md".size
   def size : Int32
     @value.includes?("*") ? @value.size : @value.size + EXT_SIZE
   end

   def ==(rhs) : Bool
     self.to_s == rhs.to_s
   end

   {% for op in [">", "<", ">=", "<="] %}
     def {{op.id}}(rhs : Path) : Bool
       self.size {{op.id}} rhs.size
     end
   {% end %}
end
